/* Name: control_battery.c
 * Project: Sun_charge
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2010-04-10
 * Tabsize: 4
 * Copyright: (c) 2010 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Sun_charge.
 *
 *   Sun_charge is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Sun_charge is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Sun_charge.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */
 
 /*
 * color of leds:
 * current - white, empty/overheat - red, normal level - green, full - blue
 * 
 */

#include <stdlib.h>
#include <avr/wdt.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include <compat/deprecated.h>
#include<avr/sleep.h>
#include <avr/eeprom.h>


#include "battery_calc.h"
#include "main.h"
#include "control_battery.h"


//-------------------------------------------------------------------------------------------------------------------------------------
uint8_t control_battery (uint16_t measureVoltage,uint8_t overHeat,uint16_t deltaTemerature)
{
static uint8_t trackerMode;
uint16_t quantedVoltage;



	quantedVoltage=(uint16_t)(measureVoltage*K_INPUT_DEVIDER);
	
if (check_Battery_Connected(quantedVoltage))
	{
		if (quantedVoltage>(uint16_t)(BATTERIES_TRESHOLD*K_ADC))
		{
		trackerMode=control_battery12V(quantedVoltage,overHeat,deltaTemerature);
		
		}
		else
		{
		trackerMode=control_battery6V(quantedVoltage,overHeat,deltaTemerature);
		
		
		}
	}
else 	
	{
	trackerMode=without_Battery_Control(quantedVoltage,overHeat);
	
	}
return trackerMode;

}

//-------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t check_Battery_Connected(uint16_t quantedVoltage) 
{

if (voltageDispersion(quantedVoltage)||(quantedVoltage>((uint16_t)(VOLTAGE_LIMIT*K_ADC)))||(quantedVoltage<(uint16_t)((BATTERY_EMPTY_6V-0.5)*K_ADC)))
return 0;

return 1;
}


//----------------------------------------------------------------------------------------------------------------------------------------------------------------------
/*
measure voltage dispersion on accumulator connector. In case we can see big voltage dispersion it means work go without accumulator's load .
*/
uint8_t voltageDispersion(uint16_t quantedVoltage)
{
static volatile uint16_t	voltageArchiv2secDuring[40];
static volatile uint16_t *p_position;
static volatile uint8_t position,start;
uint8_t	i,dispersion=0;

if (position==39) start=1;



if (position==0)
	{
	p_position=voltageArchiv2secDuring;
	position=39;
	
	}
else
	{
	position--;
	p_position++;
	}
	
*p_position=quantedVoltage;

if (start)
	{
	volatile uint16_t *p_findPosition;
	p_findPosition=voltageArchiv2secDuring;
	for (i=0;i<40;i++)
		{
		if ((((int16_t)(*p_findPosition-quantedVoltage))>((int16_t) (POSITIVE_DISPERTION_LIMIT*K_ADC)))||(((int16_t)(quantedVoltage-*p_findPosition))>((int16_t) (NEGATIVE_DISPERTION_LIMIT*K_ADC))))
			{
			dispersion=1;
			}
		p_findPosition++;
		
		}
	
	
	}
if (dispersion) return 1;

return 0;
}


//---------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
uint8_t without_Battery_Control(uint16_t quantedVoltage,uint8_t overHeat)
{
static uint8_t trackerMode;
if (quantedVoltage<(uint16_t)((VOLTAGE_LIMIT-1)*K_ADC)) 
	{
	trackerMode=WORK_TRACKER;
	sbi (PORTD,3);  // led full off 
	cbi (PORTD,4);// led normal on 
	if (overHeat==0) sbi (PORTB,6); // led empty off 
	}

if (quantedVoltage>(uint16_t)((VOLTAGE_LIMIT-0.5)*K_ADC)) 
	{
	trackerMode=STOP_TRACKER;
	cbi (PORTD,3);  // led full on 
	sbi (PORTD,4);  //led normal off 
	if (overHeat==0) sbi (PORTB,6); //led empty off 
	}
	
if (quantedVoltage>(uint16_t)((VOLTAGE_LIMIT)*K_ADC))
	{
	trackerMode=OFF_CURRENT;
	cbi (PORTD,3);  //led full on 
	sbi (PORTD,4);  //led normal off 
	if (overHeat==0) sbi (PORTB,6); //led empty off 
	}
if (quantedVoltage<(uint16_t)(BATTERY_EMPTY_6V*K_ADC)) 
	{
	if (overHeat==0) cbi (PORTB,6); //led empty on 
	sbi (PORTD,4);  //led normal off 
	sbi (PORTD,3);  //led full off  
	}


return trackerMode;
}


uint8_t control_battery12V (uint16_t quantedVoltage,uint8_t overHeat,uint16_t deltaTemerature)
{
static uint8_t trackerMode;
if (quantedVoltage<(uint16_t)(LOWER_CHARGE_VOLTAGE_12V*K_ADC)-deltaTemerature*2) trackerMode=WORK_TRACKER;
if (quantedVoltage>(uint16_t)(UPPER_CHARGE_VOLTAGE_12V*K_ADC)-deltaTemerature*2) trackerMode=OFF_CURRENT;
if (quantedVoltage>(uint16_t)(UPPER_CHARGE_VOLTAGE_12V*K_ADC)-deltaTemerature*2) cbi (PORTD,3);  //led full on 
if (quantedVoltage<(uint16_t)(LOWER_TRESHOLD_INDIC_CHARGE_12V*K_ADC)-deltaTemerature*2) sbi (PORTD,3);  // led full off

if (quantedVoltage>BATTERY_EMPTY_12V*K_ADC)
	{ 
		voltage_blink (quantedVoltage,overHeat,ALARM_LEVEL_BATTERY_12V*K_ADC,BATTERY_EMPTY_12V*K_ADC,K_blink_12V);
	}
	else
	{
	
	if (overHeat==0) cbi (PORTB,6); //led empty on 
		sbi (PORTD,4);  //led normal off  

	} 


return trackerMode;
}



uint8_t control_battery6V(uint16_t quantedVoltage,uint8_t overHeat,uint16_t deltaTemerature)
{
static uint8_t trackerMode;

if (quantedVoltage<(uint16_t)(LOWER_CHARGE_VOLTAGE_6V*K_ADC)-deltaTemerature) trackerMode=WORK_TRACKER;
if (quantedVoltage>(uint16_t)(UPPER_CHARGE_VOLTAGE_6V*K_ADC)-deltaTemerature) trackerMode=OFF_CURRENT;
if (quantedVoltage>(uint16_t)(UPPER_CHARGE_VOLTAGE_6V*K_ADC)-deltaTemerature) cbi (PORTD,3);  // led full on 
if (quantedVoltage<(uint16_t)(LOWER_TRESHOLD_INDIC_CHARGE_6V*K_ADC)-deltaTemerature) sbi (PORTD,3);  // led full off

if (quantedVoltage>BATTERY_EMPTY_6V*K_ADC)
	{ 
		voltage_blink (quantedVoltage,overHeat,ALARM_LEVEL_BATTERY_6V*K_ADC,BATTERY_EMPTY_6V*K_ADC,K_blink_6V);
		
	}
	else
	{
	
	if (overHeat==0) cbi (PORTB,6); // led empty on 
		sbi (PORTD,4);  // led normal off  

	} 


return trackerMode;
}



//-----------------------------------------------------------------------------------------------------------------------
void voltage_blink (uint16_t voltage,uint8_t overHeat,uint16_t discharge_level,uint16_t empty_level,double K_blink)
{
if (voltage> discharge_level)
		{ 
		if (overHeat==0) sbi (PORTB,6); // led empty off
		cbi (PORTD,4);// led normal on
		
		}
		else 
		{
			static uint8_t k;
			if (k>0)
			{
				k--;
			}
			else k=20;
			
			
		if (k>(uint8_t)((voltage-empty_level)*K_blink))
				
				{
					if (overHeat==0) cbi (PORTB,6); // led empty on
					sbi (PORTD,4);  // led normal off
				}
				else
				{
					if (overHeat==0) sbi (PORTB,6); // led empty off
					cbi (PORTD,4);// led normal on
				}
		
			
		
		}
}

//-------------------------------------------------------------------------------------------------------------------------------
/*
Because phone interrupts the current every 1 sec during 0.1-0.7 sec, dependence charge level of battery, current indicator use maximum 
detected value for indicate current level from solar panel. This variant easier for set angle of solar battery to sun.
*/
uint8_t maxDetector(int16_t aCurrent)
{
static volatile int16_t lastCurrentPoint [30];
static volatile int16_t  *p_lastCurrentPoint,*p_currentPosition;
static volatile uint8_t i;
uint8_t time_blink,k;
int16_t currentLevel=0;

			 

if (i==0)
	{
	i=29;
	p_currentPosition=lastCurrentPoint;
	}
else 
	{
	i--;
	p_currentPosition++;
	
	
	}
	
	*p_currentPosition=aCurrent;
	

	
	
	p_lastCurrentPoint=lastCurrentPoint;
	
	for (k=0;k<30;k++)
		{
		if (currentLevel<*p_lastCurrentPoint) currentLevel=*p_lastCurrentPoint;
		p_lastCurrentPoint++;
		}
		currentLevel-=5;
		if (currentLevel<0)	currentLevel=0;
		if (currentLevel>60)	currentLevel=60;
		time_blink=60-currentLevel;
		
		
		
	return time_blink;	
		
}


//---------------------------------------------------------------------------------------------------------------------------
void current_blink(int16_t aCurrent,uint8_t idleCurrent,uint8_t trackerMode)
{

static volatile uint8_t time_blink=60,h,i,j;


if (idleCurrent) sbi (PORTB,7);
else 
	{
		time_blink=maxDetector(aCurrent);
		
		
		
	if (60==time_blink)
	 {
	 sbi (PORTB,7);
	 
	 }
	else{
			if (i==0)
			{
				if (j)
				{
					cbi (PORTB,7);
					j=0;
				}
				else 
				{
				sbi (PORTB,7);
				j=1;
				}
				i=time_blink;
			}
			else i--;
		
		if ((i+time_blink)>h)
		i=i+time_blink-h;
		h=time_blink;
		if (i<0) i=0;
		if (i>60) i=60;
		}

	}
}

//---------------------------------------------------------------------------------------------------------------------------------
void overheat_blink()
{
static uint8_t n;
	if (n==0)
		{
		if (bit_is_set(PINB,6)) cbi (PORTB,6);
		else{
			sbi (PORTB,6);
			}
		n=20;
		}
	else n--;
}
