
/* Name: main.c
 * Project: Sun_charge
 * Author: Sergey Merkulov <merk71@rambler.ru>
 * Creation Date: 2010-04-10
 * Tabsize: 4
 * Copyright: (c) 2010 by Sergey Merkulov, all rights reserved.
 * License: GNU GPL v3 (see COPYING.txt) 
 *
 * This file is part of Sun_charge.
 *
 *   Sun_charge is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   Sun_charge is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with Sun_charge.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/*
General Description:
Sun_charge is embed program for ATmega8 controller that is part of Solar charger project (http://merk.wikidot.com/eng-electt-solar-charger)
Project compiled by WinAVR-20090313 (AVR-GCC 4.3.2)
Sun_chargeV1.2 make functions control of level charge SLA accumulator with environment temperature correction, detect 6/12V level pluged SLA accumulator, max power point tracker.
clk - Internal RC-oscillator 2 MHz.

*/
#include <stdlib.h>
#include <avr/wdt.h>
#include <inttypes.h>
#include <avr/io.h>
#include <avr/interrupt.h>

#include <compat/deprecated.h>
#include<avr/sleep.h>
#include <avr/eeprom.h>

#include "control_battery.h"
#include "battery_calc.h"
#include "main.h"


uint8_t signOfWrited __attribute__((section(".eeprom")));
uint16_t e_null_temperature __attribute__((section(".eeprom")));


void maximum_powerpoint_tracker (int16_t,uint8_t);
void measureNull(int16_t);


/* main interrupt loop--------------------------------------------------------------------------------------
 frequency of call 7.6 Hz
*/
INTERRUPT (SIG_OVERFLOW2)
{
wdt_reset ();

static uint8_t overHeat;
volatile uint8_t o,p,trackerMode,idleCurrent;
volatile int16_t voltage;
static volatile int16_t controlCurrent;
static volatile int16_t filter_voltage,deltaTemerature=10,filterTemperatyre=4300;

sbi (ADCSRA,ADEN);
	ADMUX=0x42;
	sbi (ADCSRA,ADSC);
	while (bit_is_set (ADCSRA,ADSC));
	
	o=ADCL;
	p=ADCH;
	voltage=p*0x100+o;
	
	if (voltage>40)
	{
	
	idleCurrent=0;
	}
	else 
	{
	
	idleCurrent=1;
	}
	
	ADMUX=0x41;
	sbi (ADCSRA,ADSC);
	while (bit_is_set (ADCSRA,ADSC));
	o=ADCL;
	p=ADCH;
	
	voltage=p*0x100+o-voltage;
	
	
	filter_voltage=filter_voltage+voltage-filter_voltage/2;
	voltage=filter_voltage/2;
	
	trackerMode=control_battery (voltage,overHeat,deltaTemerature);
	
	ADMUX=0x46;
	sbi (ADCSRA,ADSC);
	while (bit_is_set (ADCSRA,ADSC));

	o=ADCL;
	p=ADCH;
	filterTemperatyre=p*0x100+o+filterTemperatyre-filterTemperatyre/32;
	deltaTemerature=filterTemperatyre/32;
	deltaTemerature=(null_temperature-deltaTemerature)/2;

	if (deltaTemerature>12) overHeat=1;
	else overHeat=0;
	if (deltaTemerature<-200) overHeat=1;
	ADMUX=0x40;
	sbi (ADCSRA,ADSC);
	while (bit_is_set (ADCSRA,ADSC));

	o=ADCL;
	p=ADCH;
cbi (ADCSRA,ADEN);	

	controlCurrent=p*0x100+o;
	
	if (idleCurrent)
	{
		trackerMode=OFF_CURRENT;
		controlCurrent=125;
	}
	
	maximum_powerpoint_tracker (controlCurrent,trackerMode);
//	Current=0;
	
	current_blink(controlCurrent,idleCurrent,trackerMode);

if (overHeat)
	{
	overheat_blink();
		
	}



if (checkWriteNull!=0xaa) measureNull (filterTemperatyre);
	


}

//------------------------------------------------------------------------------------------------------------------------------------------------------------
/*
at first start of program after programming controller measure voltage value, because first start of program must be at 20 degrees Celsium.
*/
void measureNull (filterTemperatyre)
{
static int16_t filterTemperatyreLast;
int16_t nullValue;

if (filterTemperatyre==filterTemperatyreLast)
	{
	nullValue=filterTemperatyre/32+10;
	eeprom_write_word (&e_null_temperature,nullValue);
	eeprom_write_byte (&signOfWrited,0xaa);
	checkWriteNull=0xaa;
	null_temperature=nullValue;
	return;
	}
filterTemperatyreLast=filterTemperatyre;

}

/*--------------------------------------------------------------------------------------------------------------------------------------------------------------
 find optimal width pulse of PWM that give max load current
    ,      
 max PWM=255; */
void maximum_powerpoint_tracker (int16_t aCurrent,uint8_t trackerMode) 
{
volatile int16_t Delta_Current,Delta_Output;
static volatile int16_t SumPWM=50, Current_Last,Output_Last,Step=4;
static volatile uint8_t Fall_Flope,Rise_Slope,Count_Step;

if ((trackerMode==WORK_TRACKER)&&(aCurrent<150)&&aCurrent)
{
	Delta_Current=aCurrent-Current_Last;
	Current_Last=aCurrent;
	Delta_Output=SumPWM-Output_Last;
	Output_Last=SumPWM;

	if (((Delta_Output>0)&&(Delta_Current>0))||((Delta_Output<0)&&(Delta_Current<0)))
		{
		
		Step=4;
		if (Fall_Flope) Count_Step=4;
		Rise_Slope=1;
		Fall_Flope=0;
		if (Count_Step) Step=2;
		}
	if (((Delta_Output<0)&&(Delta_Current>0))||((Delta_Output>0)&&(Delta_Current<0)))
		{
		Step=-4;
		if (Rise_Slope) Count_Step=4;
		Fall_Flope=1;
		Rise_Slope=0;
		if (Count_Step) Step=-2;
		}
		
	if (Count_Step) Count_Step--;
	SumPWM=SumPWM+Step;
	
	if (SumPWM>250) 
		{
		SumPWM=250;
		Step=-4;
		

		}

	if (SumPWM<20) 
		{
		SumPWM=20;
		Step=4;
		
		}

	OCR1A=SumPWM; // max value 255
	

}



if (trackerMode==STOP_TRACKER)
	{
			if (SumPWM/2>30)
				OCR1A=SumPWM/2;
			else OCR1A=30;
	
	
	}

if ((trackerMode==OFF_CURRENT)||(aCurrent>149))
	{
	if	(aCurrent>119)
		{
		SumPWM=20;
		Step=4;
		}
	OCR1A=30;
	}



}




//--------------------------------------------------------------------------------------------------------------------------------------------------------------
int16_t read_null_temperature (void)
{

 
if (bit_is_clear(PIND,2))
	{
		

		eeprom_write_byte (&signOfWrited,0);

		

	}
checkWriteNull=eeprom_read_byte(&signOfWrited);
return eeprom_read_word(&e_null_temperature);
}

//--------------------------------------------------------------------------------------------------------------------

void ioinit (void)
{
uint8_t o,p;
sbi (DDRB,1);
sbi (DDRD,3);
sbi (DDRD,4);

sbi (DDRB,6);
sbi (DDRB,7);
sbi (PORTC,3);
sbi (PORTC,4);
sbi (PORTD,2);

ACSR=_BV(ACD);
TCCR1A=_BV(WGM10)|_BV(COM1A1);
TCCR1B=_BV(WGM12)|_BV(CS10);

OCR1A=15;

TCCR2=_BV(CS22)|_BV(CS20)|_BV(CS21); /* set timer 2 prescaler to 1024 */
ADCSRA=_BV(ADEN)|_BV(ADSC)|_BV(ADPS2)|_BV(ADPS0);
ADMUX=0x44;
sbi (ADCSRA,ADSC);
while (bit_is_set (ADCSRA,ADSC));
	o=ADCL;
	p=ADCH;

wdt_enable(WDTO_1S);
TIMSK=_BV(TOIE2);
null_temperature=read_null_temperature();
sbi (MCUCR,SE);
sei();
}


//---------------------------------------------------------------------------------------------------------------------
int main (void)
{

ioinit ();


for (;;)
{
wdt_reset ();
sleep_mode ();
}

return(0);
}
